---
layout: post
title: "How To Fix \"Out Of Bounds Pointer\" Error with GRUB"
---

Here's the process:

## Use `efibootmgr` to remove all boot messages.

<pre class="terminal">
$ efibootmgr
BootCurrent: 0000
Timeout: 5 seconds
BootOrder: 0000,2001,0002,2002,2003
...
$ # now take the IDs from the BootOrder line and remove each of them using the following commands
$ efibootmgr -b 0000 -B
$ efibootmgr -b 2001 -B
$ efibootmgr -b 0002 -B
$ efibootmgr -b 2002 -B
$ efibootmgr -b 2003 -B
</pre>

This tells your EFI system that there are no EFI entries to boot into.
Don't worry, this will get repopulated later.

## Remove `/boot/EFI/` and `/boot/grub/`

To remove all EFI boot images and all grub configurations, run the following command:

<pre class="terminal">
# rm -rf /boot/EFI/ /boot/grub/
</pre>

## Re-install `grub`

Re-install the GRUB bootloader, along with its config files.

<pre class="terminal">
# grub-install --efi-directory=/boot
# grub-mkconfig -o /boot/grub/grub.cfg
</pre>

## Conclusion

This should fix the issue, since it clears out all grub and EFI entries.
I had to re-install grub through my package manager, and change a few settings that I had set up for <abbr title="Full-Disk Encryption">FDE</abbr>.
If this is helpful, you can let me know by sending me an email, by [contributing to a project I work on](https://github.com/TTWNO/), or [sponsoring me to work on open-source technology](https://liberapay.com/tait/).

Happy GRUB hacking!
