---
layout: default
title: "Blind Linux Config Files"
permalink: "/blind-linux-configs/"
---

Here are some configuration files that I have found helpful for blind and visually impaired Linux users:

{% include toc.md %}

## Chromium/Electron

To enable accessibility in Chromium and Electron, use the following flags in `$XDG_CONFIG_HOME/chromium-flags.conf` and `$XDG_CONFIG_HOME/electron-flags.conf` respectively. `$XDG_CONFIG_HOME` defaults to `$HOME/.config`.

```text
--force-renderer-accessibility
--enable-caret-browsing
```

You will also want to add the following to your `~/.bashrc`:

```text
export ENABLE_ACCESSIBILITY=1
```

## Wi-Fi

There are a lot of different ways to connect to the internet on Linux, my favourite is with NetworkManager using `nmcli`.
It can be done as one command like the following (rememnber that `$` is just indicating that a command is being run):

<pre class="terminal">
$ nmcli dev wifi con "ssid_here" password "password_here" name "saved_name_for_future"
</pre>

And without a password (open network):

<pre class="terminal">
$ nmcli dev wifi con "ssid_here" name "saved_name"
</pre>

Now that's all well and good, but it's pretty long to remember, so I made a little script in my [scripts repository](https://git.tait.tech/tait/scripts/).
This has all sorts of nice tools related to making everything easier to use from the command line (which incidently *should* make it easier for blind peeps as well).

## GPG/PGP/pass

By default (on some distributions), when `gpg` asks for a private key password, it asks for it in a GUI pop-up window.
I find this annoying and it is slow on devices like the [PinebookPro](/2021/06/02/pinebook-pro/).
To disable this and have a plain prompt in your terminal instead, use the following in your `$HOME/.gnupg/gpg-agent.conf`:

```text
pinentry-program /usr/bin/pinentry-tty
```

To activate it, restart gpg-agent by killing it, then attempting to use a private key:

<pre class="terminal">
$ killall gpg-agent
# using gpg directly
$ gpg --encrypt ...
# using the pass password manager
$ pass show your/password
</pre>
